@@ -1,6 +1,9 @@
 import asyncio
 import dataclasses
+import mimetypes
+import os
 from pathlib import Path
+import re
 import threading
 import aiohttp
 from aiohttp import web
@@ -11,6 +14,8 @@ from collections import deque
 import time
 import uuid
 
+import urllib
+
 from rubpy.bot.enums import ChatKeypadTypeEnum, UpdateTypeEnum
 from rubpy.bot.filters import Filter
 from rubpy.bot.models import Keypad, InlineMessage, Update, Message
@@ -601,4 +606,76 @@ class BotClient:
     
     async def close(self):
         if not self.session.closed:
-            await self.session.close()
\ No newline at end of file
+            await self.session.close()
+
+    async def download_file(
+        self,
+        file_id: str,
+        save_as: Optional[str] = None,
+        progress: Optional[Callable[[int, int], None]] = None,
+        chunk_size: int = 65536,
+        as_bytes: bool = False
+    ) -> Union[str, bytes, None]:
+        """
+        Downloads a file by file_id.
+
+        Args:
+            file_id (str): Unique identifier of the file.
+            save_as (Optional[str]): File path to save. Ignored if as_bytes=True.
+            progress (Optional[Callable[[int, int], None]]): Optional progress callback.
+            chunk_size (int): Download chunk size in bytes.
+            as_bytes (bool): If True, returns the content as bytes instead of saving.
+
+        Returns:
+            str: Path to saved file (if as_bytes=False)
+            bytes: Content of the file (if as_bytes=True)
+        """
+        try:
+            download_url = await self.get_file(file_id)
+        except Exception:
+            raise ValueError(f"Invalid file_id: {file_id}")
+
+        if not download_url:
+            raise ValueError(f"Invalid download file url: {download_url}")
+
+        async with aiohttp.ClientSession() as session:
+            async with session.post(download_url) as response:
+                if response.status != 200:
+                    raise Exception(f"Failed to download file: {response.status}")
+
+                content_type = response.headers.get("Content-Type", "")
+                ext = get_extension(content_type)
+                total_size = int(response.headers.get("Content-Length", 0))
+                downloaded = 0
+
+                if as_bytes:
+                    content = bytearray()
+                    async for chunk in response.content.iter_chunked(chunk_size):
+                        content.extend(chunk)
+                        downloaded += len(chunk)
+                        if progress:
+                            try:
+                                progress(downloaded, total_size)
+                            except Exception:
+                                pass
+                    return bytes(content)
+                else:
+                    if save_as is None:
+                        save_as = f"{file_id}{ext}"
+
+                    with open(save_as, "wb") as f:
+                        async for chunk in response.content.iter_chunked(chunk_size):
+                            f.write(chunk)
+                            downloaded += len(chunk)
+                            if progress:
+                                try:
+                                    progress(downloaded, total_size)
+                                except Exception:
+                                    pass
+
+                    return save_as
+
+def get_extension(content_type: str) -> str:
+    """Convert MIME type to file extension (e.g., image/jpeg -> .jpg)."""
+    ext = mimetypes.guess_extension(content_type)
+    return ext if ext else ''
\ No newline at end of file
